from django.contrib.auth.signals import user_logged_in

from django_agent_trust import trust_agent, trust_session
from django_agent_trust.models import AgentSettings


def _handle_auth_login(sender, request, user, **kwargs):
    """
    Automatically establishes agent trust if indicated by one of our
    authentication forms.
    """
    AgentSettings.objects.get_or_create(user=user)

    if getattr(user, 'otp_trust_this_agent', False):
        trust_agent(request)
    elif getattr(user, 'otp_trust_this_session', False):
        trust_session(request)


user_logged_in.connect(_handle_auth_login)



from functools import partial

from django.contrib.auth import BACKEND_SESSION_KEY
from django.contrib.auth.views import login as auth_login

from .forms import OTPAuthenticationForm, OTPTokenForm


def login(request, **kwargs):
    """
    This is just like :func:`django_otp.views.login` except that it uses our
    agent-trust-enabled forms.
    """
    user = request.user

    if user.is_anonymous() or user.is_verified():
        form = OTPAuthenticationForm
    else:
        form = partial(OTPTokenForm, user)

        # A minor hack to make django.contrib.auth.login happy
        user.backend = request.session[BACKEND_SESSION_KEY]

    kwargs['authentication_form'] = form

    return auth_login(request, **kwargs)



from functools import wraps

from django_otp.decorators import otp_required as real_otp_required
from django_agent_trust.decorators import trusted_agent_required


def otp_required(view=None, redirect_field_name='next', login_url=None, if_configured=False, accept_trusted_agent=False):
    """
    Similar to :func:`~django_otp.decorators.otp_required`, but with an extra
    argument.

    :param bool accept_trusted_agent: If ``True``, we'll accept a trusted
        agent in lieu of OTP verification. Default is ``False``.
    """
    def decorator(view_func):
        @wraps(view_func)
        def _wrapped_view(request, *args, **kwargs):
            if request.user.is_verified() or (accept_trusted_agent and request.agent.is_trusted):
                x_required = lambda v: v
            elif accept_trusted_agent:
                x_required = trusted_agent_required(redirect_field_name=redirect_field_name, login_url=login_url)
            else:
                x_required = real_otp_required(redirect_field_name=redirect_field_name, login_url=login_url, if_configured=if_configured)

            return x_required(view_func)(request, *args, **kwargs)

        return _wrapped_view

    return decorator(view) if (view is not None) else decorator



from django.conf.urls import patterns, url

from . import views


urlpatterns = patterns('',
    url(r'^login/$', 'django.contrib.auth.views.login'),
    url(r'^verify/$', 'django_otp.views.login'),
    url(r'^trust/$', 'otp_agents.views.login'),
    url(r'^logout/$', 'django.contrib.auth.views.logout'),

    url(r'^otp/$', views.otp_view),
    url(r'^otp2/$', views.otp_view_2),
    url(r'^otp_advised/$', views.otp_advised_view),
    url(r'^agent/$', views.agent_view),
)



from django.http import HttpResponse

from otp_agents.decorators import otp_required


@otp_required
def otp_view(request):
    return HttpResponse()


@otp_required()
def otp_view_2(request):
    return HttpResponse()


@otp_required(if_configured=True)
def otp_advised_view(request):
    return HttpResponse()


@otp_required(accept_trusted_agent=True)
def agent_view(request):
    return HttpResponse()



"""
This package provides subclasses of
:class:`~django_otp.forms.OTPAuthenticationForm` and
:class:`~django_otp.forms.OTPTokenForm` with an extra boolean field called
``otp_trust_agent``. The user can check this option to indicate that they'd
like to bypass OTP verification when accessing the site from the same browser
in the future.

When these forms are used with :func:`django.contrib.auth.views.login`, we will
call the appropriate django-agent-trust APIs automatically. If the trust option
is selected, the agent will be trusted persistently; if not, it will be trusted
for the current session. Views willing to accept a trusted agent in lieu of OTP
verification may then use
:func:`~django_agent_trust.decorators.trusted_agent_required` in place of
:func:`~django_otp.decorators.otp_required`.
"""
from django import forms

import django_otp.forms


class OTPAgentFormMixin(object):
    def clean_agent(self):
        user = self.get_user()

        if getattr(user, 'otp_device', None) is not None:
            if self.cleaned_data.get('otp_trust_agent'):
                user.otp_trust_this_agent = True
            else:
                user.otp_trust_this_session = True


class OTPAuthenticationForm(django_otp.forms.OTPAuthenticationForm, OTPAgentFormMixin):
    """
    Extends :class:`~django_otp.forms.OTPAuthenticationForm` with support for
    agent trust.
    """
    otp_trust_agent = forms.BooleanField(required=False, label="Trust this agent")

    def clean(self):
        cleaned_data = super(OTPAuthenticationForm, self).clean()
        self.clean_agent()

        return cleaned_data


class OTPTokenForm(django_otp.forms.OTPTokenForm, OTPAgentFormMixin):
    """
    Extends :class:`~django_otp.forms.OTPTokenForm` with support for agent
    trust.
    """
    otp_trust_agent = forms.BooleanField(required=False, label="Trust this agent")

    def clean(self):
        cleaned_data = super(OTPTokenForm, self).clean()
        self.clean_agent()

        return cleaned_data



#!/usr/bin/env python

from setuptools import setup, find_packages


setup(
    name='django-otp-agents',
    version='0.1.3',
    description="Integration of django-otp and django-agent-trust.",
    long_description=open('README').read(),
    author='Peter Sagerson',
    author_email='psagersDjwublJf@ignorare.net',
    zip_safe=False,
    packages=find_packages(),
    include_package_data=True,
    url='https://bitbucket.org/psagers/django-otp',
    license='BSD',
    install_requires=[
        'django-otp >= 0.1.7',
        'django-agent-trust',
    ],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Programming Language :: Python :: 2",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Topic :: Security",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)



import django
from django.db import IntegrityError
from django.db.models import get_app
from django.utils.unittest import skipIf

from django_otp.tests import TestCase


@skipIf(django.VERSION < (1, 4), u"Requires Django 1.4")
class OTPAgentsTestCase(TestCase):
    urls = 'otp_agents.tests.urls'

    def setUp(self):
        try:
            get_app('otp_static')
        except:
            self.skipTest(u"Requires django_otp.plugins.otp_static")

        try:
            self.alice = self.create_user('alice', 'alice')
        except IntegrityError:
            self.skipTest(u"Unable to create a test user")
        else:
            device = self.alice.staticdevice_set.create()
            device.token_set.create(token='alice1')
            device.token_set.create(token='alice2')

    def test_otp_anonymous(self):
        response = self.client.get('/otp/')

        self.assertEquals(response.status_code, 302)

    def test_otp_authenticated(self):
        self.login()
        response = self.client.get('/otp/')

        self.assertEquals(response.status_code, 302)

    def test_otp_verified(self):
        self.verify()
        response = self.client.get('/otp/')

        self.assertEquals(response.status_code, 200)

    def test_otp_trusted(self):
        self.trust(True)
        self.logout()
        self.login()
        response = self.client.get('/otp/')

        self.assertEquals(response.status_code, 302)

    def test_otp2_verified(self):
        self.verify()
        response = self.client.get('/otp2/')

        self.assertEquals(response.status_code, 200)

    def test_otp_advised_anonymous(self):
        response = self.client.get('/otp_advised/')

        self.assertEquals(response.status_code, 302)

    def test_otp_advised_unconfigured(self):
        self.alice.staticdevice_set.all().delete()
        self.login()
        response = self.client.get('/otp_advised/')

        self.assertEquals(response.status_code, 200)

    def test_otp_advised_authenticated(self):
        self.login()
        response = self.client.get('/otp_advised/')

        self.assertEquals(response.status_code, 302)

    def test_otp_advised_verified(self):
        self.verify()
        response = self.client.get('/otp_advised/')

        self.assertEquals(response.status_code, 200)

    def test_agent_anonymous(self):
        response = self.client.get('/agent/')

        self.assertEquals(response.status_code, 302)

    def test_agent_authenticated(self):
        response = self.client.get('/agent/')

        self.assertEquals(response.status_code, 302)

    def test_agent_verified(self):
        self.verify()
        response = self.client.get('/agent/')

        self.assertEquals(response.status_code, 200)

    def test_agent_trusted_session(self):
        self.trust(False)
        response = self.client.get('/agent/')

        self.assertEquals(response.status_code, 200)

    def test_agent_trusted(self):
        self.trust(True)
        self.logout()
        self.login()
        response = self.client.get('/agent/')

        self.assertEquals(response.status_code, 200)

    def test_two_step_trust(self):
        self.login()
        self.add_trust(True)
        self.logout()
        self.login()
        response = self.client.get('/agent/')

        self.assertEquals(response.status_code, 200)

    def login(self):
        params = {
            'username': 'alice',
            'password': 'alice',
        }

        response = self.client.post('/login/', params)

        self.assertEquals(response.status_code, 302)

    def verify(self):
        params = {
            'username': 'alice',
            'password': 'alice',
            'otp_token': 'alice1',
        }

        response = self.client.post('/verify/', params)

        self.assertEquals(response.status_code, 302)

    def trust(self, persist=False):
        params = {
            'username': 'alice',
            'password': 'alice',
            'otp_token': 'alice1',
            'otp_trust_agent': '1' if persist else '',
        }

        response = self.client.post('/trust/', params)

        self.assertEquals(response.status_code, 302)

    def add_trust(self, persist=False):
        params = {
            'otp_device': 'django_otp.plugins.otp_static.models.StaticDevice/1',
            'otp_token': 'alice1',
            'otp_trust_agent': '1' if persist else '',
        }

        response = self.client.post('/trust/', params)

        self.assertEquals(response.status_code, 302)

    def logout(self):
        response = self.client.post('/logout/')

        self.assertEquals(response.status_code, 200)
